from .tracer import RequestIdContext


class SocketRequestIdMiddleware:
    def __init__(self, app, prefix: str = "S-"):
        self.app = app
        self.prefix = prefix

    async def __call__(self, scope, receive, send):
        headers = dict(scope.get("headers", []))
        trace_id = headers.get(b"x-trace-id", None)
        if trace_id:
            trace_id = trace_id.decode("utf-8")
        with RequestIdContext(trace_id, self.prefix):
            await self.app(scope, receive, send)



import asyncio
import threading

import injection  # noqa
import pytest
from adaptors.socket_client import SocketClient, SocketClientConfig
from dependency import container
from frameworks.api_app import APIService
from frameworks.socket_app import SocketService
from usecases.example_service import ExampleService, InnerService
from usecases.sample import AnotherSample

from chromatrace import LoggingConfig


class TestChromatrace:
    @pytest.fixture(scope="class")
    def logging_config(self):
        return container[LoggingConfig]

    @pytest.fixture(scope="class")
    def socket_client_config(self):
        return SocketClientConfig()

    @pytest.fixture(scope="class")
    def api_service(self, logging_config):
        return container[APIService]

    @pytest.fixture(scope="class")
    def socket_service(self):
        return container[SocketService]

    @pytest.mark.asyncio
    def test_socket_service(self, socket_service):
        assert socket_service is not None

    @pytest.fixture(scope="class")
    def socket_client(self, socket_client_config):
        return SocketClient(config=socket_client_config)

    @pytest.mark.asyncio
    async def test_socket_client(self, socket_client, socket_service):
        assert socket_client is not None
        th = threading.Thread(
            target=socket_client.start_background_loop,
            args=(socket_client.client_loop,),
            daemon=True,
        )
        th.start()
        asyncio.run_coroutine_threadsafe(socket_client.run(), socket_client.client_loop)
        await asyncio.sleep(1)  # Allow time for the client to start
        th.join(timeout=1)  # Wait for thread to finish or timeout

    @pytest.mark.asyncio
    def test_api_service(self, api_service):
        assert api_service is not None
        assert api_service.logger is not None
        assert api_service.rest_application is not None

    @pytest.mark.asyncio
    async def test_example_service(self, logging_config):
        inner_service = InnerService(logging_config)
        example_service = ExampleService(logging_config, inner_service)
        assert example_service is not None
        await example_service.do_something()

    @pytest.mark.asyncio
    async def test_another_sample(self, logging_config):
        another_sample = AnotherSample(logging_config)
        assert another_sample is not None
        another_sample.do_something()
        await another_sample.consume()
        await another_sample.send_http_request_with_trace_id()



from dependency import container
from frameworks.api_app import APIService
from frameworks.socket_app import SocketService
from lagom import Singleton

from chromatrace import LoggingSettings

container[LoggingSettings] = LoggingSettings(
    application_level="Development",
    enable_tracing=True,
    ignore_nan_trace=True,
    enable_file_logging=True,
    show_process_id=True,
)
container[APIService] = Singleton(APIService)
container[SocketService] = Singleton(SocketService)



import logging
from copy import copy
from pathlib import Path
from typing import Literal, Optional

import click
from pydantic import BaseModel


class LoggingSettings(BaseModel):
    application_level: str = ""
    log_level: str = "INFO"
    log_format: str = "(%(asctime)s)-[%(levelname)s]-[%(name)s]-FILENAME:%(filename)s-FUNC:%(funcName)s-THREAD:%(threadName)s-LINE:%(lineno)d :: %(message)s"
    date_format: str = "%Y-%m-%d %H:%M:%S"
    style: str = "%"
    file_path: Path = Path("./logs/app.log")
    max_bytes: int = 500 * 1024 * 1024  # 500MB
    backup_count: int = 3  # 3 days retention
    syslog_host: Optional[str] = None
    syslog_port: Optional[int] = None
    enable_console_logging: bool = True
    enable_file_logging: bool = False
    enable_tracing: bool = True
    ignore_nan_trace: bool = True
    use_console_colored_formatter: bool = True
    use_syslog_colored_formatter: bool = False
    use_file_colored_formatter: bool = False
    show_process_id: bool = False

    def __init__(self, **data):
        super().__init__(**data)
        if self.enable_file_logging:
            if isinstance(self.file_path, str):
                self.file_path = Path(self.file_path)
            if isinstance(self.file_path, Path):
                self.file_path.parent.mkdir(parents=True, exist_ok=True)
        if self.application_level:
            self.log_format = "[%(application_level)s]-" + self.log_format
        if self.enable_tracing:
            self.log_format = "[%(trace_id)s]-" + self.log_format
        if self.show_process_id:
            self.log_format = self.log_format.replace(
                "(%(asctime)s)-", "(%(asctime)s)-[%(process)s]-"
            )


class BasicFormatter(logging.Formatter):
    TRACE_LOG_LEVEL = 5
    _level_color_format = {
        TRACE_LOG_LEVEL: lambda level_name: click.style(
            str(level_name), fg="bright_blue"
        ),
        logging.DEBUG: lambda level_name: click.style(str(level_name), fg="blue"),
        logging.INFO: lambda level_name: click.style(str(level_name), fg="green"),
        logging.WARNING: lambda level_name: click.style(str(level_name), fg="yellow"),
        logging.ERROR: lambda level_name: click.style(str(level_name), fg="red"),
        logging.CRITICAL: lambda level_name: click.style(str(level_name), fg="magenta"),
    }

    def __init__(
        self,
        fmt: str | None = None,
        datefmt: str | None = None,
        style: Literal["%", "{", "$"] = "%",
        colored: bool | None = False,
        message_splitter: str | None = "\n",
        log_splitter: str | None = "\n",
        remove_nan_trace: bool | None = False,
    ):
        super().__init__(fmt, datefmt, style)
        self.log_splitter = (
            log_splitter or ""
        )  # To add a newline after each log message
        self.message_splitter = (
            message_splitter or ""
        )  # To add a newline before each message and the log details
        self.colored = colored
        self.remove_nan_trace = remove_nan_trace

    def _color_level_name(self, level_name: str, level_no: int) -> str:
        def default(level_name: str) -> str:
            return str(level_name)  # pragma: no cover

        func = self._level_color_format.get(level_no, default)
        return func(level_name)

    def _format_message(self, message: str) -> str:
        return click.style(message, bold=True)

    def _format_name(self, name: str) -> str:
        return click.style(name, fg="bright_black")

    def _format_trace_id(self, trace_id: str) -> str:
        return click.style(trace_id, fg="cyan")

    def _format_process_id(self, process_id: int) -> str:
        return click.style(process_id, fg="black")

    def format(self, record):
        record_copy = copy(record)
        record_copy.__dict__["process"] = f'PID:{getattr(record_copy, "process", 0)}'
        trace_id = getattr(record_copy, "trace_id", "NAN")
        if self.colored:
            record_copy.__dict__["levelname"] = self._color_level_name(
                level_name=record_copy.levelname, level_no=record_copy.levelno
            )
            record_copy.__dict__["msg"] = self._format_message(record_copy.msg)
            record_copy.__dict__["name"] = self._format_name(record_copy.name)
            record_copy.__dict__["process"] = self._format_process_id(
                record_copy.process
            )
            if not (trace_id == "NAN" and self.remove_nan_trace):
                record_copy.__dict__["trace_id"] = self._format_trace_id(trace_id)

        record_copy.__dict__["msg"] = (
            self.message_splitter + record_copy.__dict__["msg"]
        )
        message = super(BasicFormatter, self).format(record_copy) + self.log_splitter
        if self.remove_nan_trace:
            message = message.replace("[NAN]-", "")
        return message


class SysLogFormatter(BasicFormatter):
    def __init__(
        self,
        message_splitter: str | None = "",
        log_splitter: str | None = "",
        *args,
        **kwargs,
    ):
        super().__init__(
            message_splitter=message_splitter,
            log_splitter=log_splitter,
            **kwargs,
        )


class ApplicationLevelFilter(logging.Filter):
    def __init__(self, application_level):
        super().__init__()
        self.application_level = application_level

    def filter(self, record):
        record.application_level = self.application_level
        return True



import logging
import multiprocessing

import socketio
import uvicorn

from chromatrace import (
    LoggingConfig,
)
from chromatrace.socketio import SocketRequestIdMiddleware
from chromatrace.uvicorn import GetLoggingConfig, UvicornLoggingSettings


class SocketServerConfig:
    def __init__(
        self,
    ):
        self.log_level = "debug"
        self.cors_allowed_origins = "*"
        self.socketio_path = "/socket.io"
        self.logger = False
        self.engineio_logger = False
        self.always_connect = True


class SocketService:
    def __init__(
        self,
        socket_config: SocketServerConfig,
        logging_config: LoggingConfig,
    ) -> None:
        self.socket_config = socket_config
        self.logger = logging_config.get_logger("Socket")
        self.logger.setLevel(logging.DEBUG)
        self.create_socket()
        self.call_backs()

    def create_socket(self):
        self.sio = socketio.AsyncServer(
            async_mode="asgi",
            cors_allowed_origins=self.socket_config.cors_allowed_origins,
            always_connect=self.socket_config.always_connect,
            logger=self.logger if self.socket_config.logger else False,
            engineio_logger=self.logger
            if self.socket_config.engineio_logger
            else False,
        )
        self.socket_application = socketio.ASGIApp(
            socketio_server=self.sio,
            socketio_path=self.socket_config.socketio_path,
        )
        self.socket_application = SocketRequestIdMiddleware(self.socket_application)

    def run(self, main_process: bool = True):
        self.logger.info("Starting Socket Service...")
        if main_process:
            uvicorn.run(
                self.socket_application,
                host="0.0.0.0",
                port=8001,
                log_level="info",
                log_config=GetLoggingConfig(
                    UvicornLoggingSettings(
                        enable_file_logging=True,
                        show_process_id=True,
                    )
                ),
            )
        else:
            self.rest_api_process = multiprocessing.Process(
                target=uvicorn.run,
                kwargs={
                    "app": self.socket_application,
                    "host": "0.0.0.0",
                    "port": 8001,
                    "log_level": "info",
                    "log_config": GetLoggingConfig(
                        UvicornLoggingSettings(
                            enable_file_logging=True,
                            show_process_id=True,
                        )
                    ),
                },
            ).start()

    def call_backs(self):
        @self.sio.on(event="connect", namespace="/")
        async def connect(sid, environ, auth):
            self.logger.info("Socket connected on main namespace. SID: %s", sid)

        @self.sio.on(event="message", namespace="/")
        async def message(sid, data: dict):
            self.logger.info(
                "Received message on main namespace. SID: %s, Message: %s", sid, data
            )
            await self.sio.emit("message", "Hearing you, clearly.", room=sid)

        @self.sio.on(event="connect", namespace="/sample")
        async def sample_connect(sid, environ, auth):
            headers = environ.get("asgi.scope").get("headers")
            headers = {k.decode(): v.decode() for k, v in dict(headers).items()}
            self.logger.info(f"Socket connected on `sample` namespace. SID: {sid}")

        @self.sio.on(event="message", namespace="/sample")
        async def sample_message(sid, data: dict):
            self.logger.info(
                "Received message on main namespace. SID: %s, Message: %s", sid, data
            )
            await self.sio.emit(
                "message", "Hearing you, again, clearly.", room=sid, namespace="/sample"
            )



from contextlib import contextmanager

import pika

from .tracer import RequestIdContext, trace_id_ctx


@contextmanager
def rabbitmq_trace_context(channel, properties):
    request_id = properties.headers.get("X-Request-ID") if properties.headers else None
    with RequestIdContext(request_id):
        yield
        if properties.headers is None:
            properties.headers = {}
        properties.headers["X-Request-ID"] = trace_id_ctx.get()

class RabbitMQMiddleware:
    def __init__(self, connection_parameters):
        self.connection = pika.BlockingConnection(connection_parameters)
        self.channel = self.connection.channel()

    def publish(self, exchange, routing_key, body, properties=None):
        if properties is None:
            properties = pika.BasicProperties()
        with rabbitmq_trace_context(self.channel, properties):
            self.channel.basic_publish(exchange=exchange, routing_key=routing_key, body=body, properties=properties)

    def consume(self, queue, on_message_callback, auto_ack=False):
        def callback(ch, method, properties, body):
            with rabbitmq_trace_context(ch, properties):
                on_message_callback(ch, method, properties, body)
        self.channel.basic_consume(queue=queue, on_message_callback=callback, auto_ack=auto_ack)
        self.channel.start_consuming()



import asyncio
import logging
import sys
import threading
import time
from typing import Dict, List, Optional

import socketio
from pydantic import BaseModel, Field

from chromatrace import LoggingConfig, LoggingSettings
from chromatrace.tracer import trace_id_ctx

logging_config = LoggingConfig(
    LoggingSettings(
        enable_tracing=True,
        file_path="./logs/app-client.log",
        enable_file_logging=True,
        show_process_id=True,
    )
)
trace_id = "123456"
trace_id_ctx.set(f"C-{trace_id}")


class SocketClientConfig(BaseModel):
    server_url: str = "http://0.0.0.0:8001"
    namespaces: List[str] = ["/"]
    headers: Optional[Dict] = Field(default=None)
    socketio_path: str = "/socket.io"
    auth: Optional[Dict] = Field(default=None)
    logger: bool = False
    engineio_logger: bool = False
    reconnection: bool = True
    reconnection_delay: int = 3
    reconnection_attempts: int = 10


class SocketClient:
    def __init__(
        self,
        config: SocketClientConfig,
    ):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.config = config
        self.logger.info("Initializing Socket Client to connect to the server.")
        self.logger.info(
            "Server URL: %s%s", self.config.server_url, self.config.socketio_path
        )
        self.sio = socketio.AsyncClient(
            handle_sigint=True,
            logger=self.logger if self.config.logger else False,
            engineio_logger=self.logger if self.config.engineio_logger else False,
            reconnection=self.config.reconnection,
            reconnection_delay=self.config.reconnection_delay,
            reconnection_attempts=self.config.reconnection_attempts,
        )

        self._client_loop = asyncio.new_event_loop()
        self.logger.info("Socket Client is initialized.")

    def start_background_loop(self, loop: asyncio.AbstractEventLoop) -> None:
        asyncio.set_event_loop(loop)
        loop.run_forever()

    @property
    def client_loop(self):
        return self._client_loop

    async def connect_to_server(self):
        try:
            self.logger.info("Connecting to the server...")
            await self.sio.connect(
                self.config.server_url,
                headers=self.config.headers,
                socketio_path=self.config.socketio_path,
                auth=self.config.auth,
                namespaces=self.config.namespaces,
            )
            self.logger.info("Connected to the server successfully.")
        except ConnectionError:
            self.logger.error("Connection failed.")
        await self.sio.wait()

    def call_backs(self):
        @self.sio.on("connect", namespace="/")
        async def connect():
            self.logger.info("Connection established to the server with handshake.")

        @self.sio.on("disconnect", namespace="/")
        def disconnect():
            self.logger.info("Server disconnected.")
            self.client_loop.stop()

        @self.sio.on("message", namespace="/")
        async def message(data):
            self.logger.info("Message from the server in `message` event: %s", data)

    async def run(self):
        self.call_backs()
        await self.connect_to_server()


def handle_user_input(base_client: SocketClient, logger: logging.Logger):
    while True:
        if not base_client.sio.connected:
            logger.info(
                "Client connection with server has been lost, exiting the client..."
            )
            sys.exit(1)

        time.sleep(0.5)  # to enhance showcase
        user_input = input('Enter a message (or "exit" to quit): ')
        try:
            if user_input.lower() == "exit":
                break
            else:
                asyncio.run_coroutine_threadsafe(
                    base_client.sio.emit(
                        event="message",
                        data={"message": user_input},
                        namespace="/",
                    ),
                    base_client.client_loop,
                )
        except KeyboardInterrupt:
            break


def create_client():
    token = "Bearer XXXXXXXXXXX"
    config = SocketClientConfig(
        headers={
            "authorization": token,
            "x-trace-id": trace_id,
        },
        auth={"Authorization": token},
    )
    base_client = SocketClient(config=config)

    # starting the client background loop
    th = threading.Thread(
        target=base_client.start_background_loop,
        args=(base_client.client_loop,),
        daemon=True,
    )
    th.start()

    return base_client


def main():
    logger = logging_config.get_logger(__name__)
    logger.info("Starting client...")
    base_client = create_client()

    # running the client in the background loop
    asyncio.run_coroutine_threadsafe(base_client.run(), base_client.client_loop)

    time.sleep(0.5)
    logger.info("Ready to check the connection status and sending messages...")
    handle_user_input(base_client, logger)

    logger.info("Connection is closing...")
    base_client.sio.disconnect()


if __name__ == "__main__":
    main()



import multiprocessing

import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from usecases import AnotherSample, ExampleService

from chromatrace import LoggingConfig
from chromatrace.fastapi import RequestIdMiddleware
from chromatrace.uvicorn import GetLoggingConfig, UvicornLoggingSettings


class APIService:
    def __init__(
        self,
        logging_config: LoggingConfig,
        example_service: ExampleService,
        another_sample: AnotherSample,
    ):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.rest_application = FastAPI()

        self.example_service = example_service
        self.another_sample = another_sample

        # Add middleware
        self.rest_application.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],  # Allows all origins
            allow_credentials=True,
            allow_methods=["*"],  # Allows all methods
            allow_headers=["*"],  # Allows all headers
        )
        self.rest_application.add_middleware(RequestIdMiddleware)

        self.do_something()
        self.routes()

    def do_something(self):
        self.logger.debug("Check something in API service")
        self.logger.info("Doing something in API service")
        self.logger.error("Something went wrong in API service")

    def run(self, main_process: bool = True):
        self.logger.info("Starting API Endpoint...")
        if main_process:
            uvicorn.run(
                self.rest_application,  # could be "main:api_app" for worker, auto reload
                host="0.0.0.0",
                port=8000,
                log_level="debug",
                log_config=GetLoggingConfig(
                    UvicornLoggingSettings(
                        enable_file_logging=True,
                        show_process_id=True,
                    )
                ),
                # factory=True, # if using "main:api_app" will required
            )
        else:
            self.rest_api_process = multiprocessing.Process(
                target=uvicorn.run,
                kwargs={
                    "app": self.rest_application,  # could be "main:api_app" for worker, auto reload
                    "host": "0.0.0.0",
                    "port": 8000,
                    "log_level": "info",
                    "log_config": GetLoggingConfig(
                        UvicornLoggingSettings(
                            enable_file_logging=True,
                            show_process_id=True,
                        )
                    ),
                    # "factory": True,  # if using "main:api_app" will required
                },
            ).start()

    def routes(self):
        @self.rest_application.get("/")
        async def read_root():
            self.logger.info("Hello World, Arg1: %s, Arg2: %s", "arg1", "arg2")
            await self.example_service.do_something()
            return {"message": "Hello World"}

        @self.rest_application.post("/post_req")
        async def post_req(data: dict):
            return {"message": "received"}

        @self.rest_application.post("/post_req_2")
        async def post_req_2():
            raise Exception("Error occurred")
            return {"message": "received"}

        @self.rest_application.get("/consume")
        async def consume():
            await self.another_sample.consume()
            return {"message": "Consuming"}

        @self.rest_application.get("/send_http_request")
        async def send_http_request():
            await self.another_sample.send_http_request_with_trace_id()
            return {"message": "Sending HTTP request"}



import asyncio
import sys

import injection  # noqa
from dependency import container
from frameworks.api_app import APIService
from frameworks.socket_app import SocketService
from usecases.example_service import ExampleService
from usecases.sample import AnotherSample

from chromatrace import LoggingConfig, LoggingSettings, tracer

sys.stdout.reconfigure(encoding="utf-8")


def api_app():
    return container[APIService].rest_application


def socket_app():
    return container[SocketService].socket_application


@tracer
def main():
    # Optional: Set global log level
    container[LoggingSettings].log_level = "DEBUG"
    logger = container[LoggingConfig].get_logger("Main")

    logger.info("Starting main")
    service = container[ExampleService]
    sample = container[AnotherSample]
    asyncio.run(service.do_something())
    sample.do_something()
    logger.info("Finished main, Run API Service")
    container[SocketService].run(main_process=False)
    container[APIService].run(main_process=True)


if __name__ == "__main__":
    main()



import logging
import logging.handlers
import queue
import socket

from .logging_settings import (
    ApplicationLevelFilter,
    BasicFormatter,
    LoggingSettings,
    SysLogFormatter,
)
from .tracer import RequestIdFilter


class LoggingConfig:
    def __init__(self, settings: LoggingSettings):
        self.settings = settings
        self._configured = False
        if self.settings.enable_tracing:
            self.request_id_filter = RequestIdFilter()
        if self.settings.application_level:
            self.application_level_filter = ApplicationLevelFilter(
                self.settings.application_level
            )

    def configure(self):
        if self._configured:
            return

        # Set root logger level
        logging.getLogger().setLevel(self.settings.log_level)

    def get_logger(self, name: str) -> logging.Logger:
        if not self._configured:
            self.configure()

        logger = logging.getLogger(name)
        logger.propagate = False  # Prevent double logging

        # Add handlers if they don't exist
        if not logger.handlers:
            self._setup_handlers(logger)

        return logger

    def _set_logger_settings(
        self,
        handler: logging.handlers,
        formatter: logging.Formatter,
        logger: logging.Logger,
    ):
        handler.setFormatter(formatter)
        if self.settings.enable_tracing:
            handler.addFilter(self.request_id_filter)
        if self.settings.application_level:
            handler.addFilter(self.application_level_filter)
        logger.addHandler(handler)

    def _setup_console_handler(self, logger: logging.Logger):
        formatter = self._get_formatter(
            colored=self.settings.use_console_colored_formatter
        )
        if self.settings.enable_console_logging:
            console_handler = logging.StreamHandler()
            self._set_logger_settings(console_handler, formatter, logger)
            return console_handler
        return

    def _setup_file_handler(self, logger: logging.Logger):
        formatter = self._get_formatter(
            colored=self.settings.use_file_colored_formatter
        )
        if self.settings.enable_file_logging:
            file_handler = logging.handlers.RotatingFileHandler(
                filename=str(self.settings.file_path),
                maxBytes=self.settings.max_bytes,
                backupCount=self.settings.backup_count,
                encoding="utf-8",
            )
            self._set_logger_settings(file_handler, formatter, logger)
            return file_handler
        return

    def _setup_syslog_handler(self, logger: logging.Logger):
        if not self.settings.syslog_host or not self.settings.syslog_port:
            return

        try:
            syslog_formatter = SysLogFormatter(
                fmt=self.settings.log_format,
                datefmt=self.settings.date_format,
                style=self.settings.style,
                colored=self.settings.use_syslog_colored_formatter,
            )

            # Create handler with socket handling
            syslog_handler = logging.handlers.SysLogHandler(
                address=(self.settings.syslog_host, self.settings.syslog_port),
                facility=logging.handlers.SysLogHandler.LOG_USER,
                socktype=socket.SOCK_DGRAM,
            )

            # Add error handler
            syslog_handler.handleError = lambda *args, **kwargs: None

            self._set_logger_settings(syslog_handler, syslog_formatter, logger)

            return syslog_handler

        except (socket.error, OSError) as e:
            # Fallback to console logging if syslog fails
            formatter = self._get_formatter(
                colored=self.settings.use_console_colored_formatter
            )
            console = logging.StreamHandler()
            console.setFormatter(formatter)
            logger.addHandler(console)
            logger.warning(f"Failed to setup syslog handler: {str(e)}")
            print(f"Failed to setup syslog handler: {str(e)}")

    def _get_formatter(
        self,
        colored: bool = False,
    ):
        return BasicFormatter(
            fmt=self.settings.log_format,
            datefmt=self.settings.date_format,
            style=self.settings.style,
            colored=colored,
            remove_nan_trace=self.settings.ignore_nan_trace,
        )

    def _setup_handlers(self, logger: logging.Logger):
        # Queue handler
        log_queue = queue.Queue()

        # Console handler
        console_handler = self._setup_console_handler(logger)

        # File handler with rotation
        file_handler = self._setup_file_handler(logger)

        # Setup syslog with error handling
        syslog_handler = self._setup_syslog_handler(logger)

        # Queue listener
        handlers = [console_handler, file_handler, syslog_handler]
        queue_listener = logging.handlers.QueueListener(
            log_queue, *handlers, respect_handler_level=True
        )
        queue_listener.start()



from django.http import HttpRequest

from .tracer import RequestIdContext, trace_id_ctx


class RequestIdMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request: HttpRequest):
        request_id = request.headers.get("X-Request-ID")
        with RequestIdContext(request_id):
            response = self.get_response(request)
            response["X-Request-ID"] = trace_id_ctx.get()
            return response



import http
import logging
import sys
from copy import copy
from typing import Literal

import click

TRACE_LOG_LEVEL = 5


class BasicFormatter(logging.Formatter):
    """
    A custom log formatter class that:

    * Outputs the LOG_LEVEL with an appropriate color.
    * If a log call includes an `extras={"color_message": ...}` it will be used
      for formatting the output, instead of the plain text message.
    """

    _level_name_colors = {
        TRACE_LOG_LEVEL: lambda level_name: click.style(
            str(level_name), fg="bright_blue"
        ),
        logging.DEBUG: lambda level_name: click.style(str(level_name), fg="blue"),
        logging.INFO: lambda level_name: click.style(
            str(level_name), fg="green", blink=True
        ),
        logging.WARNING: lambda level_name: click.style(str(level_name), fg="yellow"),
        logging.ERROR: lambda level_name: click.style(str(level_name), fg="red"),
        logging.CRITICAL: lambda level_name: click.style(str(level_name), fg="magenta"),
    }

    def __init__(
        self,
        fmt: str | None = None,
        datefmt: str | None = None,
        style: Literal["%", "{", "$"] = "%",
        use_colors: bool | None = True,
    ):
        if use_colors in (True, False):
            self.use_colors = use_colors
        else:
            self.use_colors = sys.stdout.isatty()
        super().__init__(fmt=fmt, datefmt=datefmt, style=style)

    def color_level_name(self, level_name: str, level_no: int) -> str:
        def default(level_name: str) -> str:
            return str(level_name)  # pragma: no cover

        func = self._level_name_colors.get(level_no, default)
        return func(level_name)

    def should_use_colors(self) -> bool:
        return True  # pragma: no cover

    def get_colored_process_id(self, process_id: int) -> str:
        return click.style(process_id, fg="black")

    def get_process_id_line(self, process_id: int) -> str:
        return f"PID:{process_id}"

    def formatMessage(self, record: logging.LogRecord) -> str:
        record_copy = copy(record)
        process = self.get_process_id_line(record_copy.process)
        levelname = record_copy.levelname
        if self.use_colors:
            process = self.get_colored_process_id(process)
            levelname = self.color_level_name(levelname, record_copy.levelno)
            if "color_message" in record_copy.__dict__:
                record_copy.msg = record_copy.__dict__["color_message"]
                record_copy.__dict__["message"] = record_copy.getMessage()
        record_copy.__dict__["levelname"] = levelname
        record_copy.__dict__["process"] = process
        return super().formatMessage(record_copy)


class DefaultFormatter(BasicFormatter):
    def should_use_colors(self) -> bool:
        return sys.stderr.isatty()  # pragma: no cover


class AccessFormatter(BasicFormatter):
    status_code_colors = {
        1: lambda code: click.style(str(code), fg="bright_white"),
        2: lambda code: click.style(str(code), fg="green"),
        3: lambda code: click.style(str(code), fg="yellow"),
        4: lambda code: click.style(str(code), fg="red"),
        5: lambda code: click.style(str(code), fg="bright_red"),
    }
    method_colors = {
        "GET": lambda m: click.style(m, fg="green"),
        "POST": lambda m: click.style(m, fg="cyan"),
        "PUT": lambda m: click.style(m, fg="blue"),
        "DELETE": lambda m: click.style(m, fg="red"),
        "PATCH": lambda m: click.style(m, fg="magenta"),
        "HEAD": lambda m: click.style(m, fg="yellow"),
        "OPTIONS": lambda m: click.style(m, fg="white"),
        "TRACE": lambda m: click.style(m, fg="bright_black"),
    }

    def get_status_code(self, status_code: int) -> str:
        try:
            status_phrase = http.HTTPStatus(status_code).phrase
        except ValueError:
            status_phrase = ""
        status_and_phrase = f"{status_code} {status_phrase}"
        if self.use_colors:

            def default(code: int) -> str:
                return status_and_phrase

            func = self.status_code_colors.get(status_code // 100, default)
            return func(status_and_phrase)
        return status_and_phrase

    def get_colored_method(self, method: str) -> str:
        def default(method: str) -> str:
            return click.style(method, fg="white")

        func = self.method_colors.get(method.upper(), default)
        return func(method)

    def get_colored_path(self, path: str) -> str:
        return click.style(path, fg="bright_black")

    def get_colored_client_address(self, client_address: str) -> str:
        return click.style(client_address, fg="bright_black")

    def formatMessage(self, record: logging.LogRecord) -> str:
        record_copy = copy(record)
        (
            client_addr,
            method,
            full_path,
            http_version,
            status_code,
        ) = record_copy.args
        status_detail = self.get_status_code(int(status_code))
        if self.use_colors:
            method = self.get_colored_method(method)
            full_path = self.get_colored_path(full_path)
            client_addr = self.get_colored_client_address(client_addr)
        request_detail = f"{method} {full_path} HTTP/{http_version}"
        record_copy.__dict__.update(
            {
                "client_address": client_addr,
                "request_detail": request_detail,
                "status_detail": status_detail,
            }
        )
        return super().formatMessage(record_copy)



import asyncio
import logging
import os
from io import StringIO

import pytest
from fastapi import FastAPI
from fastapi.testclient import TestClient

from chromatrace import (
    LoggingConfig,
    LoggingSettings,
    RequestIdContext,
    trace_id_ctx,
    tracer,
)
from chromatrace.fastapi import RequestIdMiddleware as FastAPIMiddleware
from chromatrace.logging_settings import ApplicationLevelFilter, BasicFormatter
from chromatrace.socketio import SocketRequestIdMiddleware
from chromatrace.tracer import RequestIdFilter


class TestLoggingConfiguration:
    @pytest.fixture(scope="class")
    def settings(self):
        return LoggingSettings(
            log_level="INFO",
            enable_console_logging=True,
            enable_file_logging=True,
            enable_tracing=True,
            application_level="TestLoggingConfiguration",
            file_path="test.log",
        )

    def test_basic_logging_setup(self, settings):
        config = LoggingConfig(settings)
        logger = config.get_logger("configuration")
        logger.setLevel(settings.log_level)
        assert isinstance(logger, logging.Logger)
        assert logger.level == logging.INFO

    def test_file_logging(self, settings):
        config = LoggingConfig(settings)
        logger = config.get_logger("configuration")
        logger.info("Test message")
        assert os.path.exists(settings.file_path)

    def test_tracing_enabled(self, settings):
        config = LoggingConfig(settings)
        logger = config.get_logger("configuration")
        with RequestIdContext("test-id"):
            logger.info("Test message")
            assert trace_id_ctx.get() == "R-test-id"


class TestRequestIdMiddleware:
    @pytest.fixture
    def fastapi_app(self):
        app = FastAPI()
        app.add_middleware(FastAPIMiddleware)

        @app.get("/")
        async def root():
            return {"trace_id": trace_id_ctx.get()}

        return app

    def test_fastapi_middleware(self, fastapi_app):
        client = TestClient(fastapi_app)
        response = client.get("/", headers={"X-Request-ID": "test-id"})
        assert response.status_code == 200
        assert response.headers["X-Request-ID"] is not None

    def test_socketio_middleware(self):
        async def test_app(scope, receive, send):
            assert trace_id_ctx.get() is not None

        app = SocketRequestIdMiddleware(test_app)
        scope = {"headers": [(b"x-trace-id", b"test-id")]}

        async def run_test():
            await app(scope, None, None)

        asyncio.run(run_test())


class TestTracerDecorator:
    @tracer
    async def async_function(self):
        return trace_id_ctx.get()

    @tracer
    def sync_function(self):
        return trace_id_ctx.get()

    def test_sync_tracer(self):
        result = self.sync_function()
        assert result.startswith("T-")

    @pytest.mark.asyncio
    async def test_async_tracer(self):
        result = await self.async_function()
        assert result.startswith("T-")


class TestFormatters:
    @pytest.fixture(scope="class")
    def config(self):
        return LoggingConfig(
            LoggingSettings(
                application_level="TestFormatters",
                enable_console_logging=False,
                enable_tracing=True,
                ignore_nan_trace=True,
            )
        )

    @pytest.fixture
    def stream(self):
        return StringIO()

    @pytest.fixture
    def logger(self, config, stream):
        # Clear all existing loggers
        logger = config.get_logger("formatter-test")

        # Add StreamHandler directly to logger
        handler = logging.StreamHandler(stream)
        handler.setFormatter(
            BasicFormatter(
                fmt=config.settings.log_format,
                datefmt=config.settings.date_format,
                style=config.settings.style,
                colored=True,
                remove_nan_trace=True,
            )
        )
        handler.addFilter(RequestIdFilter())
        handler.addFilter(ApplicationLevelFilter(config.settings.application_level))
        logger.addHandler(handler)
        return logger

    def test_colored_output(self, logger, stream):
        with RequestIdContext("test-id"):
            logger.info("Test message")
            output = stream.getvalue()

        assert "\033[" in output  # ANSI color codes
        assert "Test message" in output
        assert "test-id" in output

    def test_ignore_nan_trace(self, logger, stream):
        trace_id_ctx.set("NAN")
        logger.info("Test message with no trace...")
        output = stream.getvalue()
        assert "NAN" not in output, "NAN trace should be removed"


class TestContextManagement:
    def test_nested_context(self):
        with RequestIdContext("outer"):
            assert trace_id_ctx.get() == "R-outer"
            with RequestIdContext("inner"):
                assert trace_id_ctx.get() == "R-inner"
            assert trace_id_ctx.get() == "R-outer"

    def test_request_id_filter(self):
        filter = RequestIdFilter()
        record = logging.LogRecord("test", logging.INFO, "", 0, "msg", (), None)
        filter.filter(record)
        assert hasattr(record, "trace_id")



from pathlib import Path
from typing import Optional

from pydantic import BaseModel

__all__ = ["GetLoggingConfig", "UvicornLoggingSettings"]


class UvicornLoggingSettings(BaseModel):
    log_level: str = "INFO"
    file_path: Path = Path("./log/uvicorn_app.log")
    max_bytes: int = 500 * 1024 * 1024  # 500MB
    backup_count: int = 3  # 3 days retention
    syslog_host: Optional[str] = None
    syslog_port: Optional[int] = None
    enable_console_logging: bool = True
    enable_file_logging: bool = False
    use_console_colored_formatter: bool = True
    use_syslog_colored_formatter: bool = False
    use_file_colored_formatter: bool = False
    show_process_id: bool = False

    def __init__(self, **data):
        super().__init__(**data)
        if self.enable_file_logging:
            if isinstance(self.file_path, str):
                self.file_path = Path(self.file_path)
            if isinstance(self.file_path, Path):
                self.file_path.parent.mkdir(parents=True, exist_ok=True)


def GetLoggingConfig(logging_settings: UvicornLoggingSettings):
    default_fmt = "[%(levelname)s]: %(message)s"
    access_fmt = '[%(levelname)s]: ADDRESS:(%(client_address)s) - REQUEST:"%(request_detail)s" - STATUS:%(status_detail)s'
    if logging_settings.show_process_id:
        default_fmt = "(%(asctime)s)-[%(process)s]-" + default_fmt
        access_fmt = "(%(asctime)s)-[%(process)s]-" + access_fmt
    else:
        default_fmt = "(%(asctime)s)-" + default_fmt
        access_fmt = "(%(asctime)s)-" + access_fmt
    LOGGING_CONFIG = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "default": {
                "()": "chromatrace.uvicorn.formatters.DefaultFormatter",
                "fmt": default_fmt,
                "datefmt": "%Y-%m-%d %H:%M:%S",
                "use_colors": True,
            },
            "colorless_default": {
                "()": "chromatrace.uvicorn.formatters.DefaultFormatter",
                "fmt": default_fmt,
                "datefmt": "%Y-%m-%d %H:%M:%S",
                "use_colors": False,
            },
            "access": {
                "()": "chromatrace.uvicorn.formatters.AccessFormatter",
                "fmt": access_fmt,
                "datefmt": "%Y-%m-%d %H:%M:%S",
                "use_colors": True,
            },
            "colorless_access": {
                "()": "chromatrace.uvicorn.formatters.AccessFormatter",
                "fmt": access_fmt,
                "datefmt": "%Y-%m-%d %H:%M:%S",
                "use_colors": False,
            },
        },
        "handlers": {},
        "loggers": {
            "uvicorn": {"handlers": [], "level": "INFO", "propagate": False},
            "uvicorn.error": {"handlers": [], "level": "INFO"},
            "uvicorn.access": {
                "handlers": [],
                "level": "INFO",
                "propagate": False,
            },
        },
    }
    if logging_settings.enable_console_logging:
        config = {
            "formatter": "",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stderr",
        }
        access_config = config.copy()
        config["formatter"] = "colorless_default"
        access_config["formatter"] = "colorless_access"
        if logging_settings.use_console_colored_formatter:
            config["formatter"] = "default"
            access_config["formatter"] = "access"
        LOGGING_CONFIG["handlers"]["default"] = config
        LOGGING_CONFIG["handlers"]["access"] = access_config
        LOGGING_CONFIG["loggers"]["uvicorn"]["handlers"].append("default")
        LOGGING_CONFIG["loggers"]["uvicorn.access"]["handlers"].append("access")
    if logging_settings.enable_file_logging:
        config = {
            "formatter": "",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": str(logging_settings.file_path),
            "maxBytes": logging_settings.max_bytes,
            "backupCount": logging_settings.backup_count,
            "encoding": "utf-8",
        }
        access_config = config.copy()
        config["formatter"] = "colorless_default"
        access_config["formatter"] = "colorless_access"
        if logging_settings.use_file_colored_formatter:
            config["formatter"] = "default"
            access_config["formatter"] = "access"
        LOGGING_CONFIG["handlers"]["access_file"] = access_config
        LOGGING_CONFIG["handlers"]["access_file_default"] = config
        LOGGING_CONFIG["loggers"]["uvicorn"]["handlers"].append("access_file_default")
        LOGGING_CONFIG["loggers"]["uvicorn.access"]["handlers"].append("access_file")
    if logging_settings.syslog_host and logging_settings.syslog_port:
        config = {
            "formatter": "colorless_default",
            "class": "logging.handlers.SysLogHandler",
            "address": (logging_settings.syslog_host, logging_settings.syslog_port),
        }
        config["formatter"] = "colorless_access"
        if logging_settings.use_syslog_colored_formatter:
            config["formatter"] = "access"
        LOGGING_CONFIG["handlers"]["syslog"] = config
        LOGGING_CONFIG["loggers"]["uvicorn.access"]["handlers"].append("syslog")

    return LOGGING_CONFIG



from .logging_config import LoggingConfig
from .logging_settings import LoggingSettings
from .tracer import (
    RequestIdContext,
    get_trace_id,
    trace_id_ctx,
    tracer,
)

__all__ = [
    "LoggingConfig",
    "LoggingSettings",
    "RequestIdContext",
    "get_trace_id",
    "trace_id_ctx",
    "tracer",
]



import asyncio
import logging

from chromatrace import LoggingConfig, get_trace_id


class AnotherSample:
    def __init__(self, logging_config: LoggingConfig):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.background_task = set()
        self.logger.setLevel(logging.DEBUG)

    def do_something(self):
        self.logger.debug("Check something")
        self.logger.info("Doing something")
        self.logger.warning("Doing something")
        self.logger.error("Something went wrong")

    async def consume(self):
        trace_id = get_trace_id()
        self.logger.info(f"Consuming on {trace_id}")
        self.background_task.add(asyncio.create_task(self.run()))

    async def run(self):
        self.logger.info("Starting background task")
        self.logger.info("Background task started")
        await asyncio.sleep(5)
        self.logger.info("Stopping background task")
        self.logger.info("Background task stopped")

    async def send_http_request_with_trace_id(self):
        self.logger.info("Sending HTTP request with trace id")
        trace_id = get_trace_id()
        self.background_task.add(asyncio.create_task(self.send_http_request(trace_id)))

    async def send_http_request(self, trace_id: str):
        self.logger.info(f"Sending HTTP request with trace id: {trace_id}")
        await asyncio.sleep(5)
        self.logger.info("HTTP request completed")
